#!/usr/bin/env python
# -*- coding: utf-8 -*-
import time

from pynamodb.exceptions import GetError

from db_paribhasha import credentials as creds
from db_paribhasha.dynamo import dynamo_constants as db_cnts
from db_paribhasha.dynamo import dynamo_helpers as db_helpers
from db_paribhasha.error_handler import DynamoDBError
from db_paribhasha.dynamo.iteration_metadata import IterationMetadata



def create_table():
    """
    Creates table if table doesn't exist
    """

    if IterationMetadata.exists():
        raise OperationNotPerformedError("Table already exists")
    IterationMetadata.create_table(
        read_capacity_units=DYNAMO_PRELOGIN_READ_UNITS,
        write_capacity_units=DYNAMO_PRELOGIN_WRITE_UNITS)


def modify_iteration_status(iter_id, calculated_id, status):
    """ 
        Modifies the existing row in the table
        Parameters
        ---------
        iter_id: IterationMetadata_dict
            pynamo_object dictionary
        Returns
        -------
        Dictionary
            return the dictionary of the pynamo object

    """
    modified_ts = int(round(time.time() * 1000))
    iteration_metadata = IterationMetadata.get(str(iter_id), str(calculated_id))
    iteration_metadata.status = str(status)
    iteration_metadata.modified_ts = modified_ts
    db_helpers.save_dynamo_object(iteration_metadata)
    return as_dict(iteration_metadata)


def add_iteration_status(iter_id, status, algo_name, parent_type, parent_id):
    """
    Adds an algorithm status on the basis of the parameters

    Parameters
    ---------
    iter_id: Integer
        ID of the iteration
    status: string
        It contains the detial state of the algorithm
    algo_name: string
        Algorithm name which is currently executing
    parent_type : string
        Whether it is a workflow or a run
    parent_id : integer
        Workflow or run id

    Returns
    -------
    Dictionary
        return the dictionary of the pynamo object
    """
    created_ts = int(round(time.time() * 1000))
    calculated_id = parent_type + '-' + str(parent_id) + '-' + str(created_ts)
    iteration_status =\
        IterationMetadata(iter_id=iter_id, status=status,
                          created_ts=created_ts,
                          modified_ts=created_ts,
                          algo_name=algo_name,
                          parent_id=parent_id,
                          parent_type=parent_type,
                          calculated_id=calculated_id)
    db_helpers.save_dynamo_object(iteration_status)
    return as_dict(iteration_status)


def get_iteration_status(iteration_id, algo_name):
    """
    Query to get the details of the current running algorithm

    Parameters
    ---------
    parent_id: Integer
        Run or Workflow id
    algo_name: string
        Algorithm name which is currently executing

    Returns
    -------
    response: List
        List of dictonaries containing the latest response of the algorithm.
    """
    iteration_metadata_request = IterationMetadata.query(iteration_id, IterationMetadata.algo_name == str(algo_name))
    results = []
    for result in iteration_metadata_request:
        results.append(as_dict(result))
    return results



def as_dict(iteration_metadata_query):
    """
    Convert the pynamo response object to a dictionary

    Parameters
    ---------
    iteration_metadata_query: Pynamo Object
        Pynamo response object from query
    Returns
    -------
    response: Dictionary
        Converted pynamo object to dictionary
    """
    return {
        'iter_id': iteration_metadata_query.iter_id,
        'created_ts': iteration_metadata_query.created_ts,
        'modified_ts': iteration_metadata_query.modified_ts,
        'status': iteration_metadata_query.status,
        'algo_name': iteration_metadata_query.algo_name,
        'parent_type': iteration_metadata_query.parent_type,
        'parent_id': iteration_metadata_query.parent_id,
        'calculated_id': iteration_metadata_query.calculated_id
    }