#!/usr/bin/env python
# -*- coding: utf-8 -*-

from builtins import str
import time

from db_paribhasha.dynamo import dynamo_helpers as db_helpers
from db_paribhasha.dynamo.user_profile import UserProfile
from db_paribhasha.error_handler import (DynamoDBError,
                                         OperationNotPerformedError)
from pynamodb.exceptions import GetError


def create_table():
    """
    Creates table if table doesn't exist
    """

    if UserProfile.exists():
        raise OperationNotPerformedError("Table already exists")
    UserProfile.create_table(read_capacity_units=1, write_capacity_units=1)


def create_user_profile_metadata(user_id, metadata_info):
    """
    Adds user profile metadata

    Parameters
    ----------
    user_id : String/Int
        User ID for the profile
    metadata_info : Dict
        The info to store

    Returns
    -------
    user_profile: pynamo_object
        Pynamo object containing the profile
    """

    created_ts = int(round(time.time() * 1000))
    user_profile = UserProfile(
        created_ts=created_ts,
        user_id=str(user_id),
        metadata_info=metadata_info)
    db_helpers.save_dynamo_object(user_profile)
    return user_profile


def get_user_profile(user_profile_id):
    """
    Gets user's metadata for the profile ID

    Parameters
    ----------
    user_profile_id : String
        User id profile to fetch

    Returns
    -------
    pynamo object
        Pynamo object containing info
    """
    return UserProfile.get(user_profile_id)


def get_all_user_profile_info(user_id):
    """
    Gets all the user profile inforomation for a given User ID

    Parameters
    ----------
    user_id : String/Int
        User ID to get all profile info

    Raises
    ------
    DynamoDBError
        If profile is not found

    Returns
    -------
    user_profiles: pynamo_object
        Pynamo object containing all data
    """
    try:
        user_profiles = UserProfile.query(str(user_id))
    except GetError as err:
        raise DynamoDBError(err)
    return user_profiles


def get_count_all_user_profile_info(user_id):
    """
    Returns the count of exisiting user profiles wrt the given user id

    Parameters
    ----------
    user_id : String/Int
        User ID

    Returns
    -------
    Int
        count of user profiles existing
    """

    return UserProfile.count(str(user_id))
